import datetime
from typing import Any
from uuid import UUID

from pydantic_core import InitErrorDetails

from canvas_sdk.effects.note.base import NoteOrAppointmentABC
from canvas_sdk.v1.data import NoteType, Patient
from canvas_sdk.v1.data.note import NoteTypeCategories


class Note(NoteOrAppointmentABC):
    """
    Effect to create a visit note.

    Attributes:
        note_type_id (UUID | str): The ID of the note type.
        datetime_of_service (datetime.datetime): The date and time of the service.
        patient_id (str): The ID of the patient.
    """

    class Meta:
        effect_type = "NOTE"

    note_type_id: UUID | str
    datetime_of_service: datetime.datetime
    patient_id: str

    def _get_error_details(self, method: Any) -> list[InitErrorDetails]:
        """
        Validates the note type category and returns a list of error details if validation fails.

        Args:
            method (Any): The method being validated.

        Returns:
            list[InitErrorDetails]: A list of error details for invalid note type categories.
        """
        errors = super()._get_error_details(method)

        note_type_category = (
            NoteType.objects.values_list("category", flat=True).filter(id=self.note_type_id).first()
        )

        if not note_type_category:
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Note type with ID {self.note_type_id} does not exist.",
                    self.note_type_id,
                )
            )
        elif note_type_category in (
            NoteTypeCategories.APPOINTMENT,
            NoteTypeCategories.SCHEDULE_EVENT,
            NoteTypeCategories.MESSAGE,
            NoteTypeCategories.LETTER,
        ):
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Visit note type cannot be of type: {note_type_category}.",
                    self.note_type_id,
                )
            )

        if not Patient.objects.filter(id=self.patient_id).exists():
            errors.append(
                self._create_error_detail(
                    "value",
                    f"Patient with ID {self.patient_id} does not exist.",
                    self.patient_id,
                )
            )

        return errors


__exports__ = ("Note",)



import dataclasses
from abc import ABC, abstractmethod
from typing import Any


@dataclasses.dataclass
class ResponseOption:
    """A response option for a question."""

    TYPE_TEXT = "TXT"
    TYPE_INTEGER = "INT"
    TYPE_RADIO = "SING"
    TYPE_CHECKBOX = "MULT"

    def __init__(self, dbid: int, name: str, code: str, value: str) -> None:
        self.dbid: int = dbid
        self.name: str = name
        self.code: str = code
        self.value: str = value

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, ResponseOption):
            return (
                self.dbid == other.dbid
                and self.name == other.name
                and self.code == other.code
                and self.value == other.value
            )
        return False

    def __repr__(self) -> str:
        return f"ResponseOption({self.dbid=!r}, {self.name=!r}, {self.code=!r}, {self.value=!r})"


class BaseQuestion(ABC):
    """Base class for questions."""

    type: str

    def __init__(
        self, name: str, label: str, coding: dict[str, str], options: list[ResponseOption]
    ) -> None:
        self.name: str = name
        self.label: str = label
        self.coding: dict[str, str] = coding
        self.options: list[ResponseOption] = options
        self.response: Any | None = None

    def __repr__(self) -> str:
        return f"Question({self.name=!r}, {self.label=!r}, {self.type=!r}, {self.options=!r}, {self.response=!r})"

    @abstractmethod
    def add_response(self, *args: Any, **kwargs: Any) -> None:
        """Record a response for the question.

        Subclasses will override this to perform type-specific validation.
        """
        raise NotImplementedError("Subclasses must implement this method.")


class TextQuestion(BaseQuestion):
    """A question that expects a text response."""

    type = ResponseOption.TYPE_TEXT

    def add_response(self, /, text: str) -> None:
        """For a text question, the response must be a string."""
        if not isinstance(text, str):
            raise ValueError(
                f"Response for a text question must be a string. Question: {self.label}"
            )
        self.response = text


class IntegerQuestion(BaseQuestion):
    """A question that expects an integer response."""

    type = ResponseOption.TYPE_INTEGER

    def add_response(self, /, integer: int) -> None:
        """For an integer question, the response must be convertible to int."""
        try:
            self.response = int(integer)
        except (ValueError, TypeError) as e:
            raise ValueError(
                f"Response for IntegerQuestion must be convertible to an integer. Question: {self.label}"
            ) from e


class RadioQuestion(BaseQuestion):
    """A question that expects a single-choice response."""

    type = ResponseOption.TYPE_RADIO

    def add_response(self, /, option: ResponseOption) -> None:
        """Record a radio response.

        Expects a single keyword argument 'option' of type ResponseOption.
        Validates that the option's value is among the allowed options.
        """
        if option not in self.options:
            raise ValueError(
                f"Invalid response option for radio question '{self.label}'. Allowed options: {self.options}"
            )

        self.response = option.dbid


class CheckboxQuestion(BaseQuestion):
    """A question that expects a multiple-choice response."""

    type = ResponseOption.TYPE_CHECKBOX

    def add_response(
        self, /, option: ResponseOption, selected: bool = True, comment: str = ""
    ) -> None:
        """Record a checkbox response.

        Validates that:
          - 'option' is a ResponseOption.
          - 'selected' is a bool (defaults to True if not provided).
          - 'comment' is a string (defaults to empty string if not provided).
        The response is stored as a list of dictionaries.
        """
        if option not in self.options:
            raise ValueError(
                f"Invalid response option for checkbox question '{self.label}'. Allowed values: {self.options}"
            )

        if not self.response:
            self.response = []
        self.response.append(
            {"text": option.name, "value": option.dbid, "comment": comment, "selected": selected}
        )


__exports__ = (
    "ResponseOption",
    "BaseQuestion",
    "TextQuestion",
    "IntegerQuestion",
    "RadioQuestion",
    "CheckboxQuestion",
)



import inspect
from collections.abc import Callable
from pathlib import Path
from types import FrameType
from typing import Any

from canvas_sdk.utils.metrics import measured
from settings import PLUGIN_DIRECTORY


@measured
def plugin_only(func: Callable[..., Any]) -> Callable[..., Any]:
    """Decorator to restrict a function's execution to plugins only."""

    def wrapper(*args: Any, **kwargs: Any) -> Any:
        current_frame = inspect.currentframe()
        caller = current_frame.f_back if current_frame else None

        if not caller or "__is_plugin__" not in caller.f_globals:
            return None

        plugin_name = caller.f_globals["__name__"].split(".")[0]
        plugin_dir = Path(PLUGIN_DIRECTORY) / plugin_name
        kwargs["plugin_name"] = plugin_name
        kwargs["plugin_dir"] = plugin_dir.resolve()

        return func(*args, **kwargs)

    return wrapper


@measured
def is_plugin_caller(depth: int = 10, frame: FrameType | None = None) -> tuple[bool, str | None]:
    """Check if a function is called from a plugin."""
    current_frame = frame or inspect.currentframe()
    caller = current_frame.f_back if current_frame else None

    if not caller:
        return False, None

    if "__is_plugin__" not in caller.f_globals:
        if depth > 0:
            return is_plugin_caller(frame=caller, depth=depth - 1)
        else:
            return False, None

    module = caller.f_globals.get("__name__")
    qualname = caller.f_code.co_qualname

    return True, f"{module}.{qualname}"


__exports__ = ()



from __future__ import annotations

from typing import TYPE_CHECKING, Any

import arrow
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.db.models import TextChoices

from canvas_sdk.v1.data.common import (
    AddressState,
    AddressType,
    AddressUse,
    ContactPointState,
    ContactPointSystem,
    ContactPointUse,
)

if TYPE_CHECKING:
    from django_stubs_ext.db.models.manager import RelatedManager


class SexAtBirth(TextChoices):
    """SexAtBirth."""

    FEMALE = "F", "female"
    MALE = "M", "male"
    OTHER = "O", "other"
    UNKNOWN = "UNK", "unknown"
    BLANK = "", ""


class PatientSettingConstants:
    """PatientSettingConstants."""

    LAB = "lab"
    PHARMACY = "pharmacy"
    IMAGING_CENTER = "imagingCenter"
    CONTACT_METHOD = "contactMethod"
    PREFERRED_SCHEDULING_TIMEZONE = "preferredSchedulingTimezone"


class Patient(models.Model):
    """A class representing a patient."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patient_001"

    id = models.CharField(max_length=32, db_column="key")
    dbid = models.BigIntegerField(db_column="dbid", primary_key=True)
    first_name = models.CharField()
    last_name = models.CharField()
    birth_date = models.DateField()
    sex_at_birth = models.CharField(choices=SexAtBirth.choices)
    created = models.DateTimeField()
    modified = models.DateTimeField()
    prefix = models.CharField()
    suffix = models.CharField()
    middle_name = models.CharField()
    maiden_name = models.CharField()
    nickname = models.CharField()
    sexual_orientation_term = models.CharField()
    sexual_orientation_code = models.CharField()
    gender_identity_term = models.CharField()
    gender_identity_code = models.CharField()
    preferred_pronouns = models.CharField()
    biological_race_codes = ArrayField(models.CharField())
    last_known_timezone = models.CharField()
    mrn = models.CharField()
    active = models.BooleanField()
    deceased = models.BooleanField()
    deceased_datetime = models.DateTimeField()
    deceased_cause = models.TextField()
    deceased_comment = models.TextField()
    other_gender_description = models.CharField()
    social_security_number = models.CharField()
    administrative_note = models.TextField()
    clinical_note = models.TextField()
    mothers_maiden_name = models.CharField()
    multiple_birth_indicator = models.BooleanField()
    birth_order = models.BigIntegerField()
    default_location_id = models.BigIntegerField()
    default_provider_id = models.BigIntegerField()
    user = models.ForeignKey("v1.CanvasUser", on_delete=models.DO_NOTHING, null=True)

    settings: RelatedManager[PatientSetting]

    @classmethod
    def find(cls, id: str) -> Patient:
        """Find a patient by id."""
        return cls._default_manager.get(id=id)

    def __str__(self) -> str:
        return f"{self.first_name} {self.last_name}"

    def age_at(self, time: arrow.Arrow) -> float:
        """Given a datetime, returns what the patient's age would be at that datetime."""
        age = float(0)
        birth_date = arrow.get(self.birth_date)
        if birth_date.date() < time.date():
            age = time.datetime.year - birth_date.datetime.year
            if time.datetime.month < birth_date.datetime.month or (
                time.datetime.month == birth_date.datetime.month
                and time.datetime.day < birth_date.datetime.day
            ):
                age -= 1

            current_year = birth_date.shift(years=age)
            next_year = birth_date.shift(years=age + 1)
            age += (time.date() - current_year.date()) / (next_year.date() - current_year.date())
        return age

    def get_setting(self, name: str) -> Any:
        """Returns a patient setting value by name."""
        try:
            return self.settings.get(name=name).value
        except PatientSetting.DoesNotExist:
            return None

    @property
    def preferred_pharmacy(self) -> dict[str, str] | None:
        """Returns the patient's preferred pharmacy."""
        pharmacy_setting = self.get_setting(PatientSettingConstants.PHARMACY) or {}
        if isinstance(pharmacy_setting, list):
            for pharmacy in pharmacy_setting:
                if pharmacy.get("default", False):
                    return pharmacy
            return None
        return pharmacy_setting


class PatientContactPoint(models.Model):
    """A class representing a patient contact point."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patientcontactpoint_001"

    id = models.UUIDField()
    dbid = models.BigIntegerField(primary_key=True)
    system = models.CharField(choices=ContactPointSystem.choices)
    value = models.CharField()
    use = models.CharField(choices=ContactPointUse.choices)
    use_notes = models.CharField()
    rank = models.IntegerField()
    state = models.CharField(choices=ContactPointState.choices)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="telecom", null=True
    )
    has_consent = models.BooleanField()
    last_verified = models.DateTimeField
    verification_token = models.CharField()
    opted_out = models.BooleanField()


class PatientAddress(models.Model):
    """A class representing a patient address."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patientaddress_001"

    id = models.UUIDField()
    dbid = models.BigIntegerField(primary_key=True)
    line1 = models.CharField()
    line2 = models.CharField()
    city = models.CharField()
    district = models.CharField()
    state_code = models.CharField()
    postal_code = models.CharField()
    use = models.CharField(choices=AddressUse.choices)
    type = models.CharField(choices=AddressType.choices)
    longitude = models.FloatField()
    latitude = models.FloatField()
    start = models.DateField()
    end = models.DateField()
    country = models.CharField()
    state = models.CharField(choices=AddressState.choices)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="addresses", null=True
    )

    def __str__(self) -> str:
        return f"id={self.id}"


class PatientExternalIdentifier(models.Model):
    """A class representing a patient external identifier."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patientexternalidentifier_001"

    id = models.UUIDField()
    dbid = models.BigIntegerField(primary_key=True)
    created = models.DateTimeField()
    modified = models.DateTimeField()
    patient = models.ForeignKey(
        "v1.Patient", related_name="external_identifiers", on_delete=models.DO_NOTHING, null=True
    )
    use = models.CharField()
    identifier_type = models.CharField()
    system = models.CharField()
    value = models.CharField()
    issued_date = models.DateField()
    expiration_date = models.DateField()

    def __str__(self) -> str:
        return f"id={self.id}"


class PatientSetting(models.Model):
    """PatientSetting."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patientsetting_001"

    dbid = models.BigIntegerField(primary_key=True)
    created = models.DateTimeField()
    modified = models.DateTimeField()
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="settings", null=True
    )
    name = models.CharField()
    value = models.JSONField()


class PatientMetadata(models.Model):
    """A class representing Patient Metadata."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_patientmetadata_001"

    dbid = models.BigIntegerField(primary_key=True)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="metadata", null=True
    )
    key = models.CharField(max_length=255)
    value = models.CharField(max_length=255)


__exports__ = (
    "SexAtBirth",
    "PatientSettingConstants",
    "Patient",
    "PatientContactPoint",
    "PatientAddress",
    "PatientExternalIdentifier",
    "PatientSetting",
    "PatientMetadata",
    # not defined here but used by current plugins
    "ContactPointState",
    "ContactPointSystem",
)



import os
import sys
from pathlib import Path
from urllib import parse

from dotenv import load_dotenv
from env_tools import env_to_bool

load_dotenv()

BASE_DIR = Path(__file__).resolve().parent.resolve()
FOURTEEN_DAYS = 60 * 60 * 24 * 14

ENV = os.getenv("ENV", "development")
IS_PRODUCTION = ENV == "production"
IS_PRODUCTION_CUSTOMER = env_to_bool("IS_PRODUCTION_CUSTOMER", IS_PRODUCTION)
IS_TESTING = env_to_bool("IS_TESTING", "pytest" in sys.argv[0] or sys.argv[0] == "-c")
IS_SCRIPT = env_to_bool("IS_SCRIPT", "plugin_runner.py" not in sys.argv[0])
CUSTOMER_IDENTIFIER = os.getenv("CUSTOMER_IDENTIFIER", "local")
APP_NAME = os.getenv("APP_NAME")

INTEGRATION_TEST_URL = os.getenv("INTEGRATION_TEST_URL")
INTEGRATION_TEST_CLIENT_ID = os.getenv("INTEGRATION_TEST_CLIENT_ID")
INTEGRATION_TEST_CLIENT_SECRET = os.getenv("INTEGRATION_TEST_CLIENT_SECRET")

GRAPHQL_ENDPOINT = os.getenv("GRAPHQL_ENDPOINT", "http://localhost:8000/plugins-graphql")
REDIS_ENDPOINT = os.getenv("REDIS_ENDPOINT", f"redis://{APP_NAME}-redis:6379")
CANVAS_SDK_PLUGINS_CACHE_LOCATION = os.getenv(
    "CANVAS_SDK_PLUGINS_CACHE_LOCATION", "plugin_io_plugins_cache"
)
CANVAS_SDK_CACHE_TIMEOUT_SECONDS = int(os.getenv("CANVAS_SDK_CACHE_TIMEOUT", FOURTEEN_DAYS))


METRICS_ENABLED = env_to_bool("PLUGINS_METRICS_ENABLED", not IS_SCRIPT)

INSTALLED_APPS = [
    "canvas_sdk.v1",
]

SECRET_KEY = os.getenv(
    "SECRET_KEY",
    "This value is the key to securing signed data – it is vital you keep this secure, or attackers could use it to generate their own signed values.",
)

# Use BigAutoField for Default Primary Key field type
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

CANVAS_SDK_DB_NAME = os.getenv("CANVAS_SDK_DB_NAME", "home-app")
CANVAS_SDK_DB_USERNAME = os.getenv("CANVAS_SDK_DB_USERNAME", "app")
CANVAS_SDK_DB_PASSWORD = os.getenv("CANVAS_SDK_DB_PASSWORD", "app")
CANVAS_SDK_DB_HOST = os.getenv("CANVAS_SDK_DB_HOST", "home-app-db")
CANVAS_SDK_DB_PORT = os.getenv("CANVAS_SDK_DB_PORT", "5432")

PLUGIN_RUNNER_MAX_WORKERS = int(os.getenv("PLUGIN_RUNNER_MAX_WORKERS", 5))

if os.getenv("DATABASE_URL"):
    parsed_url = parse.urlparse(os.getenv("DATABASE_URL"))

    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "OPTIONS": {"pool": {"min_size": 2, "max_size": PLUGIN_RUNNER_MAX_WORKERS}},
            "NAME": parsed_url.path[1:],
            "USER": os.getenv("CANVAS_SDK_DATABASE_ROLE"),
            "PASSWORD": os.getenv("CANVAS_SDK_DATABASE_ROLE_PASSWORD"),
            "HOST": parsed_url.hostname,
            "PORT": parsed_url.port,
        }
    }
else:
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "OPTIONS": {"pool": {"min_size": 2, "max_size": PLUGIN_RUNNER_MAX_WORKERS}},
            "NAME": CANVAS_SDK_DB_NAME,
            "USER": CANVAS_SDK_DB_USERNAME,
            "PASSWORD": CANVAS_SDK_DB_PASSWORD,
            "HOST": CANVAS_SDK_DB_HOST,
            "PORT": CANVAS_SDK_DB_PORT,
        }
    }

AWS_ACCESS_KEY_ID = os.getenv("AWS_ACCESS_KEY_ID", "")
AWS_SECRET_ACCESS_KEY = os.getenv("AWS_SECRET_ACCESS_KEY", "")
AWS_REGION = os.getenv("AWS_REGION", "us-west-2")
MEDIA_S3_BUCKET_NAME = os.getenv("MEDIA_S3_BUCKET_NAME", "canvas-client-media")

# ONTOLOGIES_SIGNING_KEY = os.getenv("ONTOLOGIES_SIGNING_KEY", "")
PLUGIN_RUNNER_SIGNING_KEY = os.getenv("PLUGIN_RUNNER_SIGNING_KEY", "")
# SCIENCE_SIGNING_KEY = os.getenv("SCIENCE_SIGNING_KEY", "")

PLUGIN_DIRECTORY = os.getenv(
    "PLUGIN_DIRECTORY",
    (
        "/plugin-runner/custom-plugins"
        if IS_PRODUCTION
        else (BASE_DIR / "plugin_runner/tests/data/plugins").as_posix()
        if IS_TESTING
        else (BASE_DIR / "custom-plugins").as_posix()
    ),
)
PLUGINS_PUBSUB_CHANNEL = os.getenv("PLUGINS_PUBSUB_CHANNEL", default="plugins")
CHANNEL_NAME = f"{CUSTOMER_IDENTIFIER}:{PLUGINS_PUBSUB_CHANNEL}"
MANIFEST_FILE_NAME = "CANVAS_MANIFEST.json"

SECRETS_FILE_NAME = "SECRETS.json"

SENTRY_DSN = os.getenv("SENTRY_DSN")

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": False,
        "OPTIONS": {},
    },
]


if IS_TESTING:
    CACHES = {
        "default": {
            "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        },
        "plugins": {
            "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
            "KEY_PREFIX": CUSTOMER_IDENTIFIER,
            "LOCATION": "plugins_cache",
            "TIMEOUT": CANVAS_SDK_CACHE_TIMEOUT_SECONDS,
        },
    }
else:
    CACHES = {
        "plugins": {
            "BACKEND": "django.core.cache.backends.db.DatabaseCache",
            "KEY_PREFIX": CUSTOMER_IDENTIFIER,
            "LOCATION": CANVAS_SDK_PLUGINS_CACHE_LOCATION,
            "TIMEOUT": CANVAS_SDK_CACHE_TIMEOUT_SECONDS,
        }
    }



from pydantic import Field

from canvas_sdk.commands.base import _BaseCommand


class MedicationStatementCommand(_BaseCommand):
    """A class for managing a MedicationStatement command within a specific note."""

    class Meta:
        key = "medicationStatement"

    fdb_code: str | None = Field(
        default=None, json_schema_extra={"commands_api_name": "medication"}
    )
    sig: str | None = None


# how do we make sure fdb_code is a valid code?

# idea1:
# create an auto-generated enum class of all possible fdbs, then type the field as that enum
# will require releasing a new version with the new codes every year, and devs will need to update
# to make sure they have the latest version to get the right set of codes.

# idea2:
# see if we can get ValueSets to play nicely with pydantic

__exports__ = ("MedicationStatementCommand",)



from canvas_sdk.commands.base import _BaseCommand as BaseCommand


class FamilyHistoryCommand(BaseCommand):
    """A class for managing a Family History command within a specific note."""

    class Meta:
        key = "familyHistory"

    family_history: str | None = None
    relative: str | None = None
    note: str | None = None


__exports__ = ("FamilyHistoryCommand",)



from typing import Self, cast

from django.db import models
from django.db.models import TextChoices

from canvas_sdk.v1.data.base import (
    BaseModelManager,
    BaseQuerySet,
    CommittableQuerySetMixin,
    ForPatientQuerySetMixin,
    ValueSetLookupQuerySetMixin,
)


class ClinicalStatus(TextChoices):
    """Condition clinical status."""

    ACTIVE = "active", "active"
    RELAPSE = "relapse", "relapse"
    REMISSION = "remission", "remission"
    RESOLVED = "resolved", "resolved"
    INVESTIGATIVE = "investigative", "investigative"


class ConditionQuerySet(
    BaseQuerySet,
    ValueSetLookupQuerySetMixin,
    CommittableQuerySetMixin,
    ForPatientQuerySetMixin,
):
    """ConditionQuerySet."""

    def active(self) -> Self:
        """Return a queryset that filters for active conditions."""
        return self.committed().filter(clinical_status=ClinicalStatus.ACTIVE)


ConditionManager = BaseModelManager.from_queryset(ConditionQuerySet)


class Condition(models.Model):
    """Condition."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_condition_001"

    objects = cast(ConditionQuerySet, ConditionManager())

    id = models.UUIDField()
    dbid = models.BigIntegerField(primary_key=True)
    deleted = models.BooleanField()
    entered_in_error = models.ForeignKey("v1.CanvasUser", on_delete=models.DO_NOTHING, null=True)
    committer = models.ForeignKey("v1.CanvasUser", on_delete=models.DO_NOTHING, null=True)
    patient = models.ForeignKey(
        "v1.Patient", on_delete=models.DO_NOTHING, related_name="conditions", null=True
    )
    onset_date = models.DateField()
    resolution_date = models.DateField()
    clinical_status = models.CharField(choices=ClinicalStatus.choices)


class ConditionCoding(models.Model):
    """ConditionCoding."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_conditioncoding_001"

    dbid = models.BigIntegerField(primary_key=True)
    system = models.CharField()
    version = models.CharField()
    code = models.CharField()
    display = models.CharField()
    user_selected = models.BooleanField()
    condition = models.ForeignKey(
        Condition, on_delete=models.DO_NOTHING, related_name="codings", null=True
    )


__exports__ = (
    "ClinicalStatus",
    "Condition",
    "ConditionCoding",
)



from django.contrib.postgres.fields import ArrayField
from django.db import models


class BannerAlert(models.Model):
    """BannerAlert."""

    class Meta:
        managed = False
        db_table = "canvas_sdk_data_api_banneralert_001"

    dbid = models.BigIntegerField(db_column="dbid", primary_key=True)
    created = models.DateTimeField()
    modified = models.DateTimeField()
    patient = models.ForeignKey(
        "v1.Patient",
        on_delete=models.DO_NOTHING,
        related_name="banner_alerts",
        null=True,
    )
    plugin_name = models.CharField()
    key = models.CharField()
    narrative = models.CharField()
    placement = ArrayField(models.CharField())
    intent = models.CharField()
    href = models.CharField()
    status = models.CharField()


__exports__ = ("BannerAlert",)



from typing import Literal
from uuid import UUID

from django.db.models.expressions import Subquery
from pydantic import Field
from pydantic_core import InitErrorDetails

from canvas_sdk.commands.base import _BaseCommand as BaseCommand
from canvas_sdk.v1.data import Condition, Note


class ResolveConditionCommand(BaseCommand):
    """A class for managing a ResolveCondition command within a specific note."""

    class Meta:
        key = "resolveCondition"

    condition_id: UUID | str | None = Field(
        default=None, json_schema_extra={"commands_api_name": "condition"}
    )
    show_in_condition_list: bool = False
    rationale: str | None = Field(max_length=1024, default=None)

    def _get_error_details(
        self, method: Literal["originate", "edit", "delete", "commit", "enter_in_error"]
    ) -> list[InitErrorDetails]:
        errors = super()._get_error_details(method)

        if self.condition_id:
            subquery = Subquery(Note.objects.filter(id=self.note_uuid).values("patient_id")[:1])
            if (
                not Condition.objects.active()
                .filter(id=self.condition_id, patient=subquery)
                .exists()
            ):
                errors.append(
                    self._create_error_detail(
                        "value",
                        f"Condition with id {self.condition_id} does not exist.",
                        self.condition_id,
                    )
                )

        return errors


__exports__ = ("ResolveConditionCommand",)
